/* $Id: ul_factor.c,v 1.3 2007/02/20 14:48:06 dw Exp $ */

/* Copyright (c) Damian Weber <dweber@htw-saarland.de> */
/* License: BSD-style
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/
#include <stdio.h>

/* factorization of unsigned long n by trial division */
/* result in primes[] (prime factors)                 */
/*       and   expo[] (exponents)                     */
/* usage: see main program below                      */
void ul_factor(unsigned long n0, unsigned long *primes, int *expo, int *max_index)
{
	unsigned long q, n;
	int j;

	n = n0;
	j = 0;

	if (n == 1)
	{
		primes[0] = 1;
		*max_index = 0;
		return;
	}


	/* divisibility by 2 */
	if ((n&1)==0)
	{
		primes[j]=2;
		expo[j]=0;
		do
		{
			expo[j]++;
			n>>=1;
		}
		while ((n&1)==0);
		j++;
	}

	/* divisibility by (odd) q */
	q = 3;
	while (n>1 && q*q<=n)
	{
		if (n%q==0)
		{
			primes[j]=q;
			expo[j]=0;
			do
			{
				expo[j]++;
				n/=q;
			}
			while ((n%q)==0);
			j++;
		}
		q+=2;
	}
	if (n>1)
	{
		primes[j] = n;
		expo[j] = 1;
		j++;
	}
	*max_index = j - 1;
}

#if 0
void print_factor(unsigned long p, int e)
{
	if (e==0)
		return;

	if (e==1)
		printf("%lu", p);
	else
		printf("%lu^%d", p,e);
}

int main(int argc, char **argv)
{
	unsigned long f[32];			/* array of factors */
	int e[32];			/* array of factor indices */
	unsigned long n;			/* number to factor */

	int nf;			/* number of factors */
	int j;

	char input_line[80];


	if (argc >= 2) 
		strcpy(input_line,argv[1]);
	else    		
		fgets(input_line,80,stdin);

	if (!sscanf(input_line, "%lu", &n))
		return 1;

	printf("%lu=", n);
	if (n<0) 
	{
		n=-n;
		printf("-");
	}

	ul_factor(n, f, e, &nf);

	print_factor(f[0],e[0]);

	for (j = 1; j <= nf; j++)
	{
		putchar('*');
		print_factor(f[j],e[j]);
	}
	puts("");

	return 0;
}
#endif
